/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

#include "tree.h"


void BSP::build(){
  bbox.clear();
  gk::Point m;
  for (std::vector<gk::Triangle>::iterator itr = triangles.begin(); itr < triangles.end(); itr++) {
    gk::BBox curr = (*itr).bbox();
    bbox.Union(curr);
    m += (curr.getCenter() * 1.f/triangles.size());
  }

  if (triangles.size() <= 2)
    return;

  std::vector<gk::Triangle> leftObjects;
  std::vector<gk::Triangle> rightObjects;


  axe = bbox.MaximumExtent();
for (std::vector<gk::Triangle>::iterator itr = triangles.begin(); itr < triangles.end(); itr++) {
     switch(axe){
    case 0:
      if(m.x > itr->bbox().getCenter().x)
	rightObjects.push_back((*itr));
      else	
	leftObjects.push_back((*itr));
      break;
    case 1:
      if(m.y > itr->bbox().getCenter().y)
	rightObjects.push_back((*itr));
      else	
	leftObjects.push_back((*itr));
      break;
    case 2:
      if(m.z > itr->bbox().getCenter().z)
	rightObjects.push_back((*itr));
      else	
	leftObjects.push_back((*itr));      
      break;
    }
 }
// std::cout<<depth<<" L=>"<<leftObjects.size()<<" R=>"<<rightObjects.size()<<std::endl;
 if (leftObjects.size() != triangles.size() && rightObjects.size() != triangles.size()) {
   left = new BSP(leftObjects,depth + 1);
   right = new BSP(rightObjects,depth + 1);
 }  
}
bool BSP::intersect(const gk::Ray& ray, gk::Hit& hit){
  float t = ray.tmax, tmin , tmax;
  if (bbox.Intersect(ray, t, tmin, tmax)) {
    if (left != NULL || right != NULL) {
      bool leftIntersection = left->intersect(ray, hit);
      bool rightIntersection = right->intersect(ray, hit);
      return leftIntersection || rightIntersection;      
    }else
      return intersectedObject(ray, hit);
  }
  return false;
}
bool BSP::intersectedObject(const gk::Ray& ray, gk::Hit& hit){
  for (std::vector<gk::Triangle>::iterator itr = triangles.begin(); itr < triangles.end(); itr++) {	
    float t,  u , v;
    if( (*itr).Intersect(ray, hit.t, t, u, v)){
      hit.t= t;
      hit.u= u;
      hit.v= v;
      hit.p= ray(t);
      hit.object_id= (*itr).id;	  
    }
  }
  return hit.object_id != -1;
}





